local uabPitBull = PitBull:NewModule("UnitActionBars")

function uabPitBull:OnSwapUnitFrame(unit, frame, oldFrame)
    -- Resync the control that was associated with the old frame
    -- and associate it to the new frame.  Assuming this happens
    -- for all unit frames, the control frames will be re-associated
    -- properly after this method is called for all affected unit
    -- frames.

--    uab.print("Changing unitframe of '"..unit.."' from '"..oldFrame:GetName().."' to '"..frame:GetName().."'")
    local controls = uab.classes.UnitGroupControl:FindControlsReferencingFrame(oldFrame)
    
    for _, control in pairs(controls) do
        if control.frame:GetParent() == frame and control.unitframe ~= frame then
--            uab.print("    Changing control '"..control.frame:GetName().."' to reference new frame")
            control:ResetParent()
        end
    end
end

local uabPitBullFrameGroup = uabFrameGroup:NewModule("PitBull")

uabPitBullFrameGroup.defn = {
    expressions = {
        "^PitBullUnitFrame%d+$",
        "^PitBullCluster%d+UnitButton%d+$",
    },
}

uabPitBullFrameGroup.controls = {}
uabPitBullFrameGroup.defaultDB = {
    framegroups = {
        ["Player"] = {
            expression = "^player$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "player",
        },
        ["Pet"] = {
            expression = "^pet$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "pet",
        },
        ["Party"] = {
            expression = "^party%d$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Party Pet"] = {
            expression = "^partypet%d$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Raid"] = {
            expression = "^raid%d+$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Raid Pet"] = {
            expression = "^raidpet%d+$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Focus"] = {
            expression = "^focus$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Target"] = {
            expression = "^target$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "target",
        },
        ["Target of Target"] = {
            expression = "^targettarget$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettarget",
        },
        ["ToT Target"] = {
            expression = "^targettargettarget$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettargettarget",
        },
        ["Other Target"] = {
            expression = "target$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
    },
}

function uabPitBullFrameGroup:OnInitialize()
    self.super.OnInitialize(self)
end

function uabPitBullFrameGroup:OnEnable()
end

function uabPitBullFrameGroup:OnDisable()
end

function uabPitBullFrameGroup:NewActivator(activator)
    --[[
    We create 6 controls for the partyraid unit group, since
    the oRA2 frames will go into that group, and there are 6
    possible frames to be allocated for each unit (each person
    could be a player target and a MT.
    --]]
    self.controls[activator] = {}
    self:NewActivatorControls(activator, "player")
    self:NewActivatorControls(activator, "pet")
    
    --[[
    We generate 4 times the typical controls for the partyraid
    set of units.  We do this to handle the normal units, pets for
    each, and targets for the normal units and the pet targets
    --]]
    self:NewActivatorControls(activator, "partyraid")
    self:NewActivatorControls(activator, "partyraid")
    self:NewActivatorControls(activator, "partyraid")
    self:NewActivatorControls(activator, "partyraid")
    self:NewActivatorControls(activator, "target")
    self:NewActivatorControls(activator, "targettarget")
    self:NewActivatorControls(activator, "targettargettarget")
end

function uabPitBullFrameGroup:NewActivatorControls(activator, unitgroup)
    if not self.controls[activator][unitgroup] then
        self.controls[activator][unitgroup] = {}
    end

    for idx = 1, uab.constants.groupInfo[unitgroup].unitcount do
        table.insert(self.controls[activator][unitgroup], activator:NewUnitGroupControl(unitgroup))
    end
end

function uabPitBullFrameGroup:HaveFrameControl(frame, activator)
	local _, options = self:OptionsForFrame(frame)
	
	if not options then return end

	if self.controls[activator] and self.controls[activator][options.unitgroup] then
		return # self.controls[activator][options.unitgroup] > 0
	end
end

function uabPitBullFrameGroup:NextFrameControl(frame, activator)
	local _, options = self:OptionsForFrame(frame)

	if not options then return end
	
    if self.controls[activator] and self.controls[activator][options.unitgroup]
        and # self.controls[activator][options.unitgroup] > 0 then
		control = table.remove(self.controls[activator][options.unitgroup])
	end

	return control
end

function uabPitBullFrameGroup:OptionsForFrame(frame)
    if self:MatchFrame(frame, uabPitBullFrameGroup.defn) then
        local unit = SecureButton_GetModifiedUnit(frame)
        if not unit then
            unit = frame:GetUnit()
        end

        if unit then        
            for name, defn in pairs(self.db.profile.framegroups) do
                if name ~= "Other Target" then
                    if defn.expressions then
                        for exp, _ in pairs(defn.expressions) do
                            if unit:match(exp) then
                                return name, defn
                            end
                        end
                    end
                    if defn.expression then
                        if unit:match(defn.expression) then
                            return name, defn
                        end
                    end
                end
            end
    
            if unit:match(self.db.profile.framegroups["Other Target"].expression) then
                return "Other Target", self.db.profile.framegroups["Other Target"]
            end
        end
    end
end

function uabPitBullFrameGroup:CreateFrameGroupOptions()
    local args = {}

    args = {
        type = 'group',
        name = "PitBull",
        desc = "Use, anchor, scale, and match expression for PitBull frames",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of UAB action sets for PitBull frames",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                    self.core.core.main:FrameGroupsChanged()
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },
        
        },
    }

    args.args["Player"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player"], "Player", "PitBull Player")
    args.args["Player"].disabled = function() return self.db.profile.disabled end
    args.args["Player"].order = 10

    args.args["Pet"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Pet"], "Pet", "PitBull Pet")
    args.args["Pet"].disabled = function() return self.db.profile.disabled end
    args.args["Pet"].order = 20

    args.args["Party"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Party"], "Party", "PitBull Party")
    args.args["Party"].disabled = function() return self.db.profile.disabled end
    args.args["Party"].order = 30

    args.args["Party Pet"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Party Pet"], "Party Pet", "PitBull Party Pet")
    args.args["Party Pet"].disabled = function() return self.db.profile.disabled end
    args.args["Party Pet"].order = 40

    args.args["Raid"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Raid"], "Raid", "PitBull Raid")
    args.args["Raid"].disabled = function() return self.db.profile.disabled end
    args.args["Raid"].order = 50

    args.args["Raid Pet"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Raid Pet"], "Raid Pet", "PitBull Raid Pet")
    args.args["Raid Pet"].disabled = function() return self.db.profile.disabled end
    args.args["Raid Pet"].order = 60

    args.args["Focus"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Focus"], "Focus", "PitBull Focus")
    args.args["Focus"].disabled = function() return self.db.profile.disabled end
    args.args["Focus"].order = 60

    args.args["Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target"], "Target", "PitBull Target")
    args.args["Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target"].order = 70

    args.args["Target of Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target of Target"], "Target of Target", "PitBull Target of Target")
    args.args["Target of Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target of Target"].order = 80

    args.args["ToT Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["ToT Target"], "ToT Target", "PitBull ToT Target")
    args.args["ToT Target"].disabled = function() return self.db.profile.disabled end
    args.args["ToT Target"].order = 90

    args.args["Other Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Other Target"], "Other Target", "PitBull Other Target")
    args.args["Other Target"].disabled = function() return self.db.profile.disabled end
    args.args["Other Target"].order = 100

    return args
end

function uabPitBullFrameGroup:SetupLayouts()
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player"].layout, self, "Player")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Pet"].layout, self, "Pet")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Party"].layout, self, "Party")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Party Pet"].layout, self, "Party Pet")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Raid"].layout, self, "Raid")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Raid Pet"].layout, self, "Raid Pet")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target"].layout, self, "Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target of Target"].layout, self, "Target of Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["ToT Target"].layout, self, "ToT Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Other Target"].layout, self, "Other Target")
end

function uabPitBullFrameGroup:SetLayoutConfigureOptions(name, args)
    self.core.core.options.args.lookandfeel.args.framegroups.args[self.name].args[name].args.layout.args.configure.args = args
end

